import glob from 'globby';
import { LOCALE_SPECIES, getLocaleSpecies } from './getLocaleSpecies';
import { getFileNameFromFilePath, spliceImportFrom } from '../../util';
const baseDir = 'locales';
const filePath = 'initReactI18next.ts';
const tpl = 'initReactI18next.tpl';
const defaultLocale = 'zh-CN';
const i18nFilePathName = '@types/react-i18next/index.d.ts';

const writeTypesFile = async api => {
  var _api$userConfig$local;

  return api.generateFile.writeFile({
    path: i18nFilePathName,
    content: api.generateFile.renderTpl(api.generateFile.readTplFile(__dirname, 'index.d.ts.tpl'), {
      initReactI18nextPath: api.resolveConfigAliasNamePath(filePath),
      default: ((_api$userConfig$local = api.userConfig.locale) === null || _api$userConfig$local === void 0 ? void 0 : _api$userConfig$local.default) || defaultLocale
    })
  });
};

const writeFile = async (api, species) => {
  let imports = '';
  let resources = '';
  species.forEach(({
    originName,
    underlineName
  }, index) => {
    imports += `${spliceImportFrom(underlineName, `@/${baseDir}/${originName}`)}\n`;
    resources += `${index ? '\n' : ''}  '${originName}': {
    translation: ${underlineName},
  },`;
  });
  const {
    locale: localConfig
  } = api.userConfig;
  await api.generateFile.writeFile({
    path: filePath,
    content: api.generateFile.renderTpl(api.generateFile.readTplFile(__dirname, tpl), {
      imports,
      resources,
      default: localConfig === null || localConfig === void 0 ? void 0 : localConfig.default
    })
  });
  return species;
};

export default async function locale(api) {
  api.describe({
    id: 'locale',
    enable: () => !!api.userConfig.locale,
    onReCompile: enableBy => {
      if (!enableBy) {
        api.generateFile.removeTmpFile(i18nFilePathName);
        api.generateFile.removeTmpFile(filePath);
      }
    }
  });
  api.addDepsEntryExports(() => ({
    source: 'react-i18next'
  }));
  api.addImportToAppEntryFile({
    stage: -9,
    fn: () => ({
      source: api.resolveConfigAliasNamePath(filePath)
    })
  });
  api.addHook({
    key: LOCALE_SPECIES,

    async fn() {
      const files = await glob(api.resolveSrcPath(`${baseDir}/*.ts`));
      return files.map(f => {
        const originName = getFileNameFromFilePath(f);
        const underlineName = originName.replace(/-|_|\./g, '');
        return {
          originName,
          underlineName
        };
      });
    }

  });
  api.onGenerateFiles({
    stage: -1,
    fn: async () => {
      const localeSpecies = await getLocaleSpecies(api);
      await writeTypesFile(api);
      return writeFile(api, localeSpecies);
    }
  });
}