import { join } from 'path';
import { generateRoutes } from './routes';
import { generateRenderAppRouter, filePath as generateRenderAppRouterFilePath } from './renderAppRouter';
import { generateRouterHelper } from './generateRouterHelper';
const dTsFilePath = '@types/index.d.ts';
export default async function generateRouter(api) {
  api.addDepsEntryExports(() => ({
    source: 'react-router-dom'
  }));
  api.addImportToAppEntryFile(() => ({
    specifier: '{ BrowserRouter as Router }',
    source: 'react-router-dom'
  }));
  api.addImportToAppEntryFile(() => ({
    specifier: '{ RenderAppRouter }',
    source: api.resolveConfigAliasNamePath(generateRenderAppRouterFilePath.replace('.tsx', ''))
  }));
  api.addContainerRenderToAppEntryFile(() => `
function renderRouter(children?: React.ReactElement) {
  return (
    <Router>
      <RenderAppRouter />

      {children}
    </Router>
  );
}`);
  api.onGenerateFiles(async () => {
    await generateRoutes(api);
    await generateRenderAppRouter(api);
    await generateRouterHelper(api);
    await api.generateFile.copyFile(join(__dirname, '../../typings/common.d.ts'), api.resolveTmpPath(dTsFilePath));
  });
}