import { AsyncSeriesWaterfallHook } from 'tapable';
import getServicePaths from './getPaths';
import { ResolvePath } from './ResolvePath';
import { PrettierFiles } from './capability/Prettier';
import { GenerateFile } from './capability/Generate';
import { ServerPlugin } from './capability/plugin';
import { Hooks } from './capability/Hooks';
import { ApplyPluginsType } from './constants';
import { Logger } from './capability/Logger';
export class Service extends ResolvePath {
  constructor(opts) {
    var _opts$paths;

    super();
    this.userConfig = {};
    this.paths = {};
    this.configPathAliasName = '@@';
    this.methods = new Map();
    this.env = opts.env || 'development';
    this.paths = { ...opts.paths,
      ...getServicePaths({
        cwd: ((_opts$paths = opts.paths) === null || _opts$paths === void 0 ? void 0 : _opts$paths.cwd) || process.cwd(),
        env: this.env
      })
    };
    this.setUserConfig(opts.userConfig);
    this.prettier = new PrettierFiles(this.self);
    this.generateFile = new GenerateFile(this.self);
    this.plugin = new ServerPlugin(this.self);
    this.hooks = new Hooks();
    this.logger = new Logger();
  }

  get self() {
    return this;
  }

  setUserConfig(config) {
    if (config) {
      this.userConfig = config;
    }

    return this;
  }

  setEnv(env) {
    this.env = env;
    return this;
  }

  resolveConfigAliasNamePath(...p) {
    return [this.configPathAliasName, ...p].join('/');
  }

  forEachHooks(key, cb) {
    const hooks = this.hooks.find(key);

    for (const hook of hooks) {
      var _hook$isSkip;

      if (hook.pluginId && !this.plugin.enableStatus(hook.pluginId) || ((_hook$isSkip = hook.isSkip) === null || _hook$isSkip === void 0 ? void 0 : _hook$isSkip.call(hook))) {
        continue;
      }

      cb(hook);
    }
  }

  async applyPlugins(opts) {
    if (!opts.type || opts.type === ApplyPluginsType.ADD) {
      const tAdd = new AsyncSeriesWaterfallHook(['memo']);
      this.forEachHooks(opts.key, hook => {
        tAdd.tapPromise({
          name: hook.pluginId || hook.key,
          stage: hook.stage || 0,
          before: hook.before
        }, async memo => {
          const item = await hook.fn(opts.args);
          return item ? memo.concat(item) : memo;
        });
      });
      const result = await tAdd.promise(opts.initialValue || []);
      return !result.length && opts.defaultValue ? opts.defaultValue : result;
    }

    const tEvent = new AsyncSeriesWaterfallHook(['_']);
    this.forEachHooks(opts.key, hook => {
      tEvent.tapPromise({
        name: hook.pluginId || hook.key,
        stage: hook.stage || 0,
        before: hook.before
      }, async (memo = []) => {
        const item = await hook.fn(opts.args);
        return item ? memo.concat([item]) : memo;
      });
    });
    return tEvent.promise(undefined);
  }

  handleReCompile() {
    this.plugin.getPlugins().forEach(plugin => {
      var _plugin$onReCompile;

      plugin === null || plugin === void 0 ? void 0 : (_plugin$onReCompile = plugin.onReCompile) === null || _plugin$onReCompile === void 0 ? void 0 : _plugin$onReCompile.call(plugin);
    });
  }

  async run(isReCompile = true) {
    if (isReCompile) {
      this.handleReCompile();
    }

    await this.applyPlugins({
      key: 'onGenerateFiles',
      type: ApplyPluginsType.EVENT
    });
    await this.applyPlugins({
      key: 'addDepsEntryExports',
      type: ApplyPluginsType.EVENT
    });
    await this.applyPlugins({
      key: 'onGenerateAppEntryFile',
      type: ApplyPluginsType.EVENT
    });
    await this.prettier.setConsoleLog(false).run();
  }

}