import { join } from 'path';
import * as fs from 'fs-extra';
import { exec } from './childProcess';
export class PrettierFiles {
  static setConsoleLog(t) {
    this.isConsoleLog = t;
    return this;
  }

  static setRootDir(dir) {
    this.rootDir = dir;
    return this;
  }

  static addPrettierFile(file) {
    this.files.push(file);
  }

  static reset() {
    this.files = [];
  }

  static async isExistSparkBuildLintDep() {
    const pkg = await fs.readJson(join(this.rootDir, 'package.json'));
    const deps = pkg.devDependencies || pkg.dependencies;
    return (deps === null || deps === void 0 ? void 0 : deps['@spark-build/lint']) ? Promise.resolve() : Promise.reject();
  }

  static async isExistPrettierFile() {
    const rootFiles = await fs.readdir(this.rootDir);
    return new Promise((r, j) => {
      rootFiles.some(n => n.indexOf('prettier') !== -1) ? r(undefined) : j();
    });
  }

  static async canItBePrettier() {
    return this.isExistPrettierFile();
  }

  static prettier(filePath) {
    return exec(`yarn prettier --config .prettierrc.js --write ${filePath}`, this.isConsoleLog);
  }

  static async run() {
    if (!this.files.length) {
      return undefined;
    }

    return this.canItBePrettier().then(() => Promise.all([...new Set(this.files)].map(f => this.prettier(f)))).then(() => this.reset());
  }

}
PrettierFiles.files = [];
PrettierFiles.isConsoleLog = true;
PrettierFiles.rootDir = '';