"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = AppEntryFile;
exports.stage = void 0;

var _fsExtra = require("fs-extra");

var _GenerateAppEntryFile = require("./GenerateAppEntryFile");

var _constants = require("./constants");

const stage = -9;
exports.stage = stage;

function addStrictModeRender(api) {
  const isSkip = () => api.userConfig.strictMode !== true;

  api.addImportToAppEntryFile({
    isSkip,
    fn: () => ({
      specifier: '{ StrictMode }',
      source: 'react'
    })
  });
  api.addContainerRenderToAppEntryFile({
    stage: 998,
    isSkip,
    fn: () => `
function renderStrictMode(children?: React.ReactElement) {
  return <StrictMode>{children}</StrictMode>;
}`
  });
}

function addRootContainer(api) {
  api.addImportToAppEntryFile(() => ({
    specifier: 'ReactDOM',
    source: 'react-dom'
  }));
  api.addImportToAppEntryFile(() => ({
    specifier: '{ isFn }',
    source: '@spark-build/web-utils'
  }));
  api.addContainerRenderToAppEntryFile({
    stage: 999,
    fn: () => `
function ReactDOMContainer(ele: React.ReactElement) {
  ReactDOM.render(ele, document.getElementById('root'));
}`
  });
}

async function AppEntryFile(api) {
  api.registerMethod({
    name: 'onGenerateAppEntryFile'
  });
  Object.keys(_constants.registerMethodKeys).forEach(name => {
    api.registerMethod({
      name
    });
  });
  addStrictModeRender(api);
  addRootContainer(api);
  api.onGenerateAppEntryFile(async () => {
    const filePath = api.resolveSrcPath('main.tsx');
    const content = await _GenerateAppEntryFile.GenerateAppEntryFile.run(api);
    return (0, _fsExtra.outputFile)(filePath, content).then(() => api.prettier.addPrettierFile(filePath));
  });
}